@@ -27,8 +27,7 @@ PfxEntry::PfxEntry(AffixMgr* pmgr, affentry* dp)
   stripl = dp->stripl;       // length of strip string
   appndl = dp->appndl;       // length of append string
   numconds = dp->numconds;   // number of conditions to match
-  xpflg = dp->xpflg;         // cross product flag
-  utf8 = dp->utf8;           // is utf8?
+  opts = dp->opts;         // cross product flag
   // then copy over all of the conditions
   memcpy(&conds.base[0],&dp->conds.base[0],SETSIZE*sizeof(conds.base[0]));
   next = NULL;
@@ -48,20 +47,19 @@ PfxEntry::~PfxEntry()
     pmyMgr = NULL;
     appnd = NULL;
     strip = NULL;
-    if (utf8) {
+    if (opts & aeUTF8) {
         for (int i = 0; i < 8; i++) {
             if (conds.utf8.wchars[i]) free(conds.utf8.wchars[i]);
         }
     }
-    if (morphcode) free(morphcode);
-    if (contclass) free(contclass);
+    if (morphcode && !(opts & aeALIASM)) free(morphcode);
+    if (contclass && !(opts & aeALIASF)) free(contclass);
 }
 
 // add prefix to this word assuming conditions hold
 char * PfxEntry::add(const char * word, int len)
 {
-    int			cond;
-    char	        tword[MAXWORDUTF8LEN + 4];
+    char tword[MAXWORDUTF8LEN + 4];
 
     if ((len > stripl) && (len >= numconds) && test_condition(word) &&
        (!stripl || (strncmp(word, strip, stripl) == 0)) && 
@@ -83,7 +81,7 @@ inline int PfxEntry::test_condition(const char * st)
 {
     int cond;
     unsigned char * cp = (unsigned char *)st;
-    if (!utf8) { // 256-character codepage
+    if (!(opts & aeUTF8)) { // 256-character codepage
         for (cond = 0;  cond < numconds;  cond++) {
 	    if ((conds.base[*cp++] & (1 << cond)) == 0) return 0;
         }
@@ -122,10 +120,8 @@ inline int PfxEntry::test_condition(const char * st)
 // check if this prefix entry matches 
 struct hentry * PfxEntry::check(const char * word, int len, char in_compound, const FLAG needflag)
 {
-    int			cond;	// condition number being examined
     int	                tmpl;   // length of tmpword
     struct hentry *     he;     // hash entry of root word or NULL
-    unsigned char *	cp;		
     char	        tmpword[MAXWORDUTF8LEN + 4];
 
     // on entry prefix is 0 length or already matches the beginning of the word.
@@ -166,12 +162,12 @@ struct hentry * PfxEntry::check(const char * word, int len, char in_compound, co
 		}
                 
 		// prefix matched but no root word was found 
-                // if XPRODUCT is allowed, try again but now 
+                // if aeXPRODUCT is allowed, try again but now 
                 // ross checked combined with a suffix
 
-		//if ((xpflg & XPRODUCT) && in_compound) {
-		if ((xpflg & XPRODUCT)) {
-		   he = pmyMgr->suffix_check(tmpword, tmpl, XPRODUCT, (AffEntry *)this, NULL, 
+		//if ((opts & aeXPRODUCT) && in_compound) {
+		if ((opts & aeXPRODUCT)) {
+		   he = pmyMgr->suffix_check(tmpword, tmpl, aeXPRODUCT, (AffEntry *)this, NULL, 
                         0, NULL, FLAG_NULL, needflag, in_compound);
                    if (he) return he;
 		}
@@ -184,10 +180,8 @@ struct hentry * PfxEntry::check(const char * word, int len, char in_compound, co
 struct hentry * PfxEntry::check_twosfx(const char * word, int len,
     char in_compound, const FLAG needflag)
 {
-    int			cond;	// condition number being examined
     int	                tmpl;   // length of tmpword
     struct hentry *     he;     // hash entry of root word or NULL
-    unsigned char *	cp;		
     char	        tmpword[MAXWORDUTF8LEN + 4];
 
     // on entry prefix is 0 length or already matches the beginning of the word.
@@ -217,11 +211,11 @@ struct hentry * PfxEntry::check_twosfx(const char * word, int len,
 		tmpl += stripl;
 
 		// prefix matched but no root word was found 
-                // if XPRODUCT is allowed, try again but now 
+                // if aeXPRODUCT is allowed, try again but now 
                 // cross checked combined with a suffix
 
-		if ((xpflg & XPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
-		   he = pmyMgr->suffix_check_twosfx(tmpword, tmpl, XPRODUCT, (AffEntry *)this, needflag);
+		if ((opts & aeXPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
+		   he = pmyMgr->suffix_check_twosfx(tmpword, tmpl, aeXPRODUCT, (AffEntry *)this, needflag);
                    if (he) return he;
 		}
 	    }
@@ -234,9 +228,7 @@ struct hentry * PfxEntry::check_twosfx(const char * word, int len,
 char * PfxEntry::check_twosfx_morph(const char * word, int len,
          char in_compound, const FLAG needflag)
 {
-    int			cond;	// condition number being examined
     int	                tmpl;   // length of tmpword
-    unsigned char *	cp;		
     char	        tmpword[MAXWORDUTF8LEN + 4];
 
     // on entry prefix is 0 length or already matches the beginning of the word.
@@ -266,12 +258,12 @@ char * PfxEntry::check_twosfx_morph(const char * word, int len,
 		tmpl += stripl;
 
 		// prefix matched but no root word was found 
-                // if XPRODUCT is allowed, try again but now 
+                // if aeXPRODUCT is allowed, try again but now 
                 // ross checked combined with a suffix
 
-		if ((xpflg & XPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
+		if ((opts & aeXPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
 		    return pmyMgr->suffix_check_twosfx_morph(tmpword, tmpl,
-			     XPRODUCT, (AffEntry *)this, needflag);
+			     aeXPRODUCT, (AffEntry *)this, needflag);
 		}
 	    }
      }
@@ -281,10 +273,8 @@ char * PfxEntry::check_twosfx_morph(const char * word, int len,
 // check if this prefix entry matches 
 char * PfxEntry::check_morph(const char * word, int len, char in_compound, const FLAG needflag)
 {
-    int			cond;	// condition number being examined
     int	                tmpl;   // length of tmpword
     struct hentry *     he;     // hash entry of root word or NULL
-    unsigned char *	cp;		
     char	        tmpword[MAXWORDUTF8LEN + 4];
     char                result[MAXLNLEN];
     char * st;
@@ -335,11 +325,11 @@ char * PfxEntry::check_morph(const char * word, int len, char in_compound, const
 		}
 
 		// prefix matched but no root word was found 
-                // if XPRODUCT is allowed, try again but now 
+                // if aeXPRODUCT is allowed, try again but now 
                 // ross checked combined with a suffix
 
-		if ((xpflg & XPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
-		   st = pmyMgr->suffix_check_morph(tmpword, tmpl, XPRODUCT, (AffEntry *)this, 
+		if ((opts & aeXPRODUCT) && (in_compound != IN_CPD_BEGIN)) {
+		   st = pmyMgr->suffix_check_morph(tmpword, tmpl, aeXPRODUCT, (AffEntry *)this, 
                      FLAG_NULL, needflag);
                    if (st) {
                         strcat(result, st);
@@ -366,8 +356,7 @@ SfxEntry::SfxEntry(AffixMgr * pmgr, affentry* dp)
   stripl = dp->stripl;       // length of strip string
   appndl = dp->appndl;       // length of append string
   numconds = dp->numconds;   // number of conditions to match
-  xpflg = dp->xpflg;         // cross product flag
-  utf8 = dp->utf8;           // is utf8?
+  opts = dp->opts;         // cross product flag
 
   // then copy over all of the conditions
   memcpy(&conds.base[0],&dp->conds.base[0],SETSIZE*sizeof(conds.base[0]));
@@ -389,19 +378,18 @@ SfxEntry::~SfxEntry()
     pmyMgr = NULL;
     appnd = NULL;
     strip = NULL;    
-    if (utf8) {
+    if (opts & aeUTF8) {
         for (int i = 0; i < 8; i++) {
             if (conds.utf8.wchars[i]) free(conds.utf8.wchars[i]);  
         }
     }
-    if (morphcode) free(morphcode);
-    if (contclass) free(contclass);
+    if (morphcode && !(opts & aeALIASM)) free(morphcode);
+    if (contclass && !(opts & aeALIASF)) free(contclass);
 }
 
 // add suffix to this word assuming conditions hold
 char * SfxEntry::add(const char * word, int len)
 {
-    int			cond;
     char	        tword[MAXWORDUTF8LEN + 4];
 
      /* make sure all conditions match */
@@ -425,7 +413,7 @@ inline int SfxEntry::test_condition(const char * st, const char * beg)
 {
     int cond;
     unsigned char * cp = (unsigned char *) st;
-    if (!utf8) { // 256-character codepage
+    if (!(opts & aeUTF8)) { // 256-character codepage
         // Dmlki affix algorithm
 	for (cond = numconds;  --cond >= 0; ) {
 	    if ((conds.base[*--cp] & (1 << cond)) == 0) return 0;
@@ -469,7 +457,6 @@ struct hentry * SfxEntry::check(const char * word, int len, int optflags,
     AffEntry* ppfx, char ** wlst, int maxSug, int * ns, const FLAG cclass, const FLAG needflag)
 {
     int	                tmpl;		 // length of tmpword 
-    int			cond;		 // condition beng examined
     struct hentry *     he;              // hash entry pointer
     unsigned char *	cp;
     char	        tmpword[MAXWORDUTF8LEN + 4];
@@ -478,7 +465,7 @@ struct hentry * SfxEntry::check(const char * word, int len, int optflags,
     // if this suffix is being cross checked with a prefix
     // but it does not support cross products skip it
 
-    if (((optflags & XPRODUCT) != 0) && ((xpflg & XPRODUCT) == 0))
+    if (((optflags & aeXPRODUCT) != 0) && ((opts & aeXPRODUCT) == 0))
         return NULL;
 
     // upon entry suffix is 0 length or already matches the end of the word.
@@ -521,7 +508,7 @@ struct hentry * SfxEntry::check(const char * word, int len, int optflags,
                         // check conditional suffix (enabled by prefix)
                         if ((TESTAFF(he->astr, aflag, he->alen) || (ep && ep->getCont() &&
                                     TESTAFF(ep->getCont(), aflag, ep->getContLen()))) && 
-                            (((optflags & XPRODUCT) == 0) || 
+                            (((optflags & aeXPRODUCT) == 0) || 
                             TESTAFF(he->astr, ep->getFlag(), he->alen) ||
                              // enabled by prefix
                             ((contclass) && TESTAFF(contclass, ep->getFlag(), contclasslen))
@@ -541,8 +528,7 @@ struct hentry * SfxEntry::check(const char * word, int len, int optflags,
                 // obsolote stemming code (used only by the 
                 // experimental SuffixMgr:suggest_pos_stems)
 	        // store resulting root in wlst
-		} else if ((wlst) && (*ns < maxSug) && 
-		    TESTAFF("ABCUVWKLMQRSTqrst", aflag , 17)) { // XXX (HU) productive suffixes
+		} else if (wlst && (*ns < maxSug)) {
 		    int cwrd = 1;
         	    for (int k=0; k < *ns; k++) 
 			if (strcmp(tmpword, wlst[k]) == 0) cwrd = 0;
@@ -566,7 +552,6 @@ struct hentry * SfxEntry::check_twosfx(const char * word, int len, int optflags,
     AffEntry* ppfx, const FLAG needflag)
 {
     int	                tmpl;		 // length of tmpword 
-    int			cond;		 // condition beng examined
     struct hentry *     he;              // hash entry pointer
     unsigned char *	cp;
     char	        tmpword[MAXWORDUTF8LEN + 4];
@@ -576,7 +561,7 @@ struct hentry * SfxEntry::check_twosfx(const char * word, int len, int optflags,
     // if this suffix is being cross checked with a prefix
     // but it does not support cross products skip it
 
-    if ((optflags & XPRODUCT) != 0 &&  (xpflg & XPRODUCT) == 0)
+    if ((optflags & aeXPRODUCT) != 0 &&  (opts & aeXPRODUCT) == 0)
         return NULL;
 
     // upon entry suffix is 0 length or already matches the end of the word.
@@ -629,7 +614,6 @@ char * SfxEntry::check_twosfx_morph(const char * word, int len, int optflags,
     AffEntry* ppfx, const FLAG needflag)
 {
     int	                tmpl;		 // length of tmpword 
-    int			cond;		 // condition beng examined
     unsigned char *	cp;
     char	        tmpword[MAXWORDUTF8LEN + 4];
     PfxEntry* ep = (PfxEntry *) ppfx;
@@ -642,7 +626,7 @@ char * SfxEntry::check_twosfx_morph(const char * word, int len, int optflags,
     // if this suffix is being cross checked with a prefix
     // but it does not support cross products skip it
 
-    if ((optflags & XPRODUCT) != 0 &&  (xpflg & XPRODUCT) == 0)
+    if ((optflags & aeXPRODUCT) != 0 &&  (opts & aeXPRODUCT) == 0)
         return NULL;
 
     // upon entry suffix is 0 length or already matches the end of the word.
@@ -717,7 +701,7 @@ struct hentry * SfxEntry::get_next_homonym(struct hentry * he, int optflags, Aff
     while (he->next_homonym) {
         he = he->next_homonym;
         if ((TESTAFF(he->astr, aflag, he->alen) || (ep && ep->getCont() && TESTAFF(ep->getCont(), aflag, ep->getContLen()))) && 
-                            ((optflags & XPRODUCT) == 0 || 
+                            ((optflags & aeXPRODUCT) == 0 || 
                             TESTAFF(he->astr, ep->getFlag(), he->alen) ||
                              // handle conditional suffix
                             ((contclass) && TESTAFF(contclass, ep->getFlag(), contclasslen))
@@ -754,14 +738,14 @@ The structure affentry is defined as follows:
 
 struct affentry
 {
-   unsigned short aflag;  // ID used to represent the affix
-   char * strip;          // string to strip before adding affix
-   char * appnd;          // the affix string to add
-   short  stripl;         // length of the strip string
-   short  appndl;         // length of the affix string
-   short  numconds;       // the number of conditions that must be met
-   short  xpflg;          // flag: XPRODUCT- combine both prefix and suffix 
-   char   conds[SETSIZE]; // array which encodes the conditions to be met
+   unsigned short aflag;    // ID used to represent the affix
+   char * strip;            // string to strip before adding affix
+   char * appnd;            // the affix string to add
+   unsigned char stripl;    // length of the strip string
+   unsigned char appndl;    // length of the affix string
+   char numconds;           // the number of conditions that must be met
+   char opts;               // flag: aeXPRODUCT- combine both prefix and suffix 
+   char   conds[SETSIZE];   // array which encodes the conditions to be met
 };
 
 
