@@ -11,8 +11,6 @@
 using namespace std;
 #endif
 
-//extern char * mystrdup(const char * s);
-
 ManParser::ManParser() {
 }
 
@@ -22,6 +20,11 @@ ManParser::ManParser(const char * wordchars)
 	// wordcharacters[('/' + 256) % 256] = 1; // for paths
 }
 
+ManParser::ManParser(struct unicode_info2 * uconv, unsigned short * wordchars, int len)
+{
+	init(uconv, wordchars, len);
+}
+
 ManParser::~ManParser() 
 {
 }
@@ -43,7 +46,7 @@ char * ManParser::next_token()
 			}
 			// no break
 		case 2: // non word chars
-			if (is_wordchar(line[actual][head])) {
+			if (is_wordchar(line[actual] + head)) {
 				state = 3;
 				token = head;
 			} else if ((line[actual][head] == '\\') && 
@@ -53,7 +56,7 @@ char * ManParser::next_token()
 			}
 			break;
 		case 3: // wordchar
-			if (! is_wordchar(line[actual][head])) {
+			if (! is_wordchar(line[actual] + head)) {
 				state = 2;
 				char * t = (char *) malloc(head - token + 1);
 				t[head - token] = '\0';
@@ -62,11 +65,10 @@ char * ManParser::next_token()
 			}
 			break;
 		}
-		if (line[actual][head] == '\0') {
+                if (next_char(line[actual], &head)) {
 			state = 0;
 			return NULL;
 		}
-		head++;
 	}
 }
 
