@@ -57,6 +57,7 @@
 #if ENABLE_NLS
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
+//#include <langinfo.h>
 #endif
 
 #ifdef HAVE_LIBINTL_H
@@ -72,7 +73,11 @@
 #endif
 
 #ifdef HAVE_CURSES_H
+#ifdef HAVE_NCURSESW_H
+#include <ncurses.h>
+#else
 #include <curses.h>
+#endif
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
 #else
@@ -88,32 +93,6 @@ extern char * mystrdup(const char * s);
 
 enum { FMT_TEXT, FMT_LATEX, FMT_HTML, FMT_MAN, FMT_FIRST };
 
-TextParser * newParser(char * wordchars, int format, char * extension) {
-    TextParser * p = NULL;
-
-    switch (format) {	
-    case FMT_LATEX: p = new LaTeXParser(wordchars); break;
-    case FMT_HTML: p = new HTMLParser(wordchars); break;
-    case FMT_MAN: p = new ManParser(wordchars); break;
-    case FMT_FIRST: p = new FirstParser(wordchars);
-    }
-
-    if ((!p) && (extension)) {
-	if ((strcmp(extension, "html") == 0) ||
-    	    (strcmp(extension, "htm") == 0) ||
-	    (strcmp(extension, "xml") == 0)) {
-		p = new HTMLParser(wordchars);
-	} else if (((extension[0] > '0') && (extension[0] <= '9'))) {
-	    p = new ManParser(wordchars);
-	} else if ((strcmp(extension, "tex") == 0)) {
-	    p = new LaTeXParser(wordchars);
-	}
-    }
-
-    if (!p) p = new TextParser(wordchars);    
-    return p;
-}
-
 //using namespace std;
 
 struct wordlist {
@@ -125,6 +104,8 @@ struct wordlist {
 
 char * wordchars = NULL;
 int wordchars_len;
+unsigned short * wordchars_utf16 = NULL;
+int wordchars_utf16_len;
 char * dicname;
 char * privdicname;
 
@@ -143,6 +124,59 @@ int unicode = 0;
 
 // functions
 
+TextParser * newParser(char * wordchars, int format, char * extension, Hunspell * pMS) {
+    TextParser * p = NULL;
+
+    if (pMS->get_utf_conv()) {
+        switch (format) {	
+        case FMT_LATEX: p = new LaTeXParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_HTML: p = new HTMLParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_MAN: p = new ManParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_FIRST: p = new FirstParser(wordchars);
+        }
+    } else {
+        switch (format) {	
+        case FMT_LATEX: p = new LaTeXParser(wordchars); break;
+        case FMT_HTML: p = new HTMLParser(wordchars); break;
+        case FMT_MAN: p = new ManParser(wordchars); break;
+        case FMT_FIRST: p = new FirstParser(wordchars);
+        }
+    }
+
+    if ((!p) && (extension)) {
+	if ((strcmp(extension, "html") == 0) ||
+    	    (strcmp(extension, "htm") == 0) ||
+	    (strcmp(extension, "xml") == 0)) {
+                if (pMS->get_utf_conv()) {
+                    p = new HTMLParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                } else {
+		    p = new HTMLParser(wordchars);
+                }
+	} else if (((extension[0] > '0') && (extension[0] <= '9'))) {
+                if (pMS->get_utf_conv()) {
+	            p = new ManParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                } else {
+	            p = new ManParser(wordchars);
+                }
+	} else if ((strcmp(extension, "tex") == 0)) {
+                if (pMS->get_utf_conv()) {
+                    p = new LaTeXParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                } else {
+	            p = new LaTeXParser(wordchars);
+                }
+	}
+    }
+    if (!p) {
+        if (pMS->get_utf_conv()) {
+            p = new TextParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+        } else {
+            p = new TextParser(wordchars);    
+        }
+    }
+    return p;
+}
+
+
 #ifdef LOG
 void log(char * message)
 {
@@ -252,7 +286,7 @@ int pos;
 int bad;
 int lineno = 0;
 
-TextParser * parser = newParser(wordchars, format, NULL);
+TextParser * parser = newParser(wordchars, format, NULL, pMS);
 
 if ((filter_mode == NORMAL) || (filter_mode == PIPE)) {
     if (filter_mode == NORMAL) {
@@ -287,7 +321,7 @@ nextline: while(fgets(buf, MAXLNLEN, fileid)) {
 	}
     case '-': {
 		delete parser;
-		parser = newParser(wordchars, format, NULL);
+		parser = newParser(wordchars, format, NULL, pMS);
 		break;
 	}
     case '@': { putdic(buf+1,pMS); break; }
@@ -459,6 +493,7 @@ if (parser) delete(parser);
 
 #ifndef W32
 
+#ifdef HAVE_CURSES_H
 #ifdef HAVE_READLINE
 static char * rltext;
 
@@ -483,7 +518,6 @@ static int rl_escape (int count, int key)
 }
 #endif
 
-#ifdef HAVE_CURSES_H
 void expand_tab(char * dest, char * src, int limit) {
 	int i = 0;
 	for(int j = 0; (i < limit) && (src[j] != '\0') && (src[j] != '\r'); j++) {
@@ -892,7 +926,7 @@ void interactive_interface(Hunspell * pMS, char * filename, int format)
 
     TextParser * parser;
     char * extension = basename(filename, '.');
-    parser = newParser(wordchars, format, extension);
+    parser = newParser(wordchars, format, extension, pMS);
 
     char * tempname = (char *) malloc(strlen(filename) + strlen(TEMPNAME) + 1);
     strcpy(tempname, filename);
@@ -960,6 +994,7 @@ int main(int argc, char** argv)
 #ifdef HAVE_LOCALE_H
 	setlocale(LC_ALL, "");
 	textdomain("hunspell");
+        //if (strcmp(nl_langinfo(CODESET), "UTF-8") == 0) utf8 = 1;
 #endif
 #endif
 
@@ -1117,6 +1152,7 @@ int main(int argc, char** argv)
 
 	wordchars = mystrdup(pMS->get_wordchars());
 	wordchars_len = strlen(wordchars);
+        wordchars_utf16 = pMS->get_wordchars_utf16(&wordchars_utf16_len);
 
 	if (arg_files==-1) {
 		pipe_interface(pMS, format, stdin);
